<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Entities;

use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calendar\AllocationCalendarInterface;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Log\TaskLogInterface;
use Zeedhi\DevOps\Util\Events\Audit_with_id\AuditAble;
use Zeedhi\DevOps\Util\Events\Organization_with_id\OrganizationAble;


class Allocation
    extends \Zeedhi\DevOps\Model\Projectmanagement\Entities\Base\Allocation
    implements AuditAble, OrganizationAble, AllocationCalendarInterface, TaskLogInterface
{
    /**
     * Table name to which this class belongs.
     */
    const TABLE = "ALLOCATION";

    const TYPE_ASSIGNER  = 'A';
    const TYPE_REPORTER  = 'R';
    const TYPE_FOLLOWER  = 'F';
    const TYPE_SPONSOR   = 'S';

    const RELEVANT = 'Y';
    const NOT_RELEVANT = 'N';
    /**
     * Returns the table name related to this entity.
     * @return string
     */
    public function getTableName(){
        return self::TABLE;
    }

    /**
     * Returns all resource's calendars.
     * @return array
     */
    public function getAllCalendars()
    {
        return $this->getResource()->getAllCalendars();
    }

    public function validateIsNotAllowEndTask()
    {
        return $this->getTask()->validateIsNotAllowEndTask();
    }

    public function setTaskStatus(TaskStatus $taskStatus = NULL) {
        if(is_null($taskStatus)){
            $taskStatus = Utilities::getTaskStatusNotInitialized();
        }
        if ($taskStatus
            && $this->validateIsNotAllowEndTask()
            && ($taskStatus->getControl() == TaskStatus::TASK_STATUS_CONTROL_CONCLUDED
                || $taskStatus->getUsedInTask() == TaskStatus::USED_IN_TASK_NO)
        ) {
            $taskStatus = Utilities::getTaskStatusPaused();
        }
        $this->taskStatus = $taskStatus;
    }
}